#include <iostream>
#include <vector>
#include <set>
#include <map>
#include <cmath>
#include <algorithm>
#include <iomanip>

using namespace std;

const double eps = 1e-5;
const int64_t INF = 1e18;

struct coor {
    int x = 0, y = 0, z = 0;
};

double dist_points(coor a, coor b) {
    return sqrt((a.x - b.x) * (a.x - b.x) + (a.y - b.y) * (a.y - b.y) + (a.z - b.z) * (a.z - b.z));
}

int dist_point_line(coor p, coor f, coor s) {
    if (!(p.x >= min(f.x, s.x) && p.x <= max(f.x, s.x) && p.y >= min(f.y, s.y) && p.y <= max(f.y, s.y) &&
        p.z >= min(f.z, s.z) && p.z <= max(f.z, s.z))) {
        return 1;
    }
    double a = dist_points(p, f);
    double b = dist_points(p, s);
    double c = dist_points(f, s);
    if (abs(c - a - b) < eps) {
        return 0;
    }
    double per = (a + b + c) / 2;
    double sq = sqrt(per * (per - a) * (per - b) * (per - c));
    double h = 2 * sq / c;
    if (h - 1 > eps) {
        return 1;
    }
    return 0;
}

int n;
vector<coor> a;
vector<vector<pair<int, double>>> g;
double ans_dist = 0;
vector<int> ans_way;

void dejkstra(int s, int e) {
    set<pair<double, int>> st;
    vector<double> d(n);
    for (int i = 0; i < n; ++i) {
        d[i] = INF;
    }
    vector<int> p(n);
    d[s] = 0;
    p[s] = -1;
    st.insert({0, s});
    while (!st.empty()) {
        int v = st.begin()->second;
        st.erase(st.begin());
        //cout << v << endl;
        //cout << g.size() << endl;
        //cout << g[v].size() << endl;
        for (int i = 0; i < g[v].size(); ++i) {
            //cout << '*';
            int to = g[v][i].first;
            double len = g[v][i].second;
            //cout << to << endl;
            //cout << d[v] << endl;
            //cout << d[to] << endl;
            //cout << '!';
            //return;
            if (d[v] + 10000000 + len + to < d[to]) {
                //cout << '!';
                if (st.find({d[to], to}) != st.end())
                    st.erase({d[to], to});
                d[to] = d[v] + 10000000 + len;
                p[to] = v;
                st.insert({d[to], to});
            }
        }
    }
    //cout << d[e] << endl;
    int64_t dint = d[e];
    ans_dist += d[e] - dint / 10000000 * 10000000;
    vector<int> res;
    res.push_back(e);
    while (e != s) {
        e = p[e];
        res.push_back(e);
    }
    for (int i = res.size() - 2; i >= 0; --i) {
        ans_way.push_back(res[i]);
    }
}

int main() {
    ios_base::sync_with_stdio(0);
    cin.tie(0), cout.tie(0);
    cin >> n;
    n++;
    a.resize(n);
    g.resize(n);
    for (int i = 1; i < n; ++i) {
        cin >> a[i].x >> a[i].y >> a[i].z;
    }
    for (int v = 0; v < n; ++v) {
        for (int u = v + 1; u < n; ++u) {
            if (dist_points(a[v], a[u]) - 20 > eps) {
                continue;
            }
            int flag = 1;
            for (int p = 0; p < n; ++p) {
                if (p == v || p == u) {
                    continue;
                }
                if (dist_point_line(a[p], a[v], a[u]) == 0) {
                    flag = 0;
                    break;
                }
            }
            if (flag) {
                double dist = dist_points(a[v], a[u]);
                g[v].push_back({u, dist});
                if (v != 0) {
                    g[u].push_back({v, dist});
                }
            }
        }
    }
    /*for (int i = 0; i < n; ++i) {
        cout << i << ':' << endl;
        for (auto x : g[i]) {
            cout << x.first << ' ' << x.second << endl;
        }
    }*/
    double d1 = dist_points(a[0], a[1]);
    double d2 = dist_points(a[0], a[2]);
    double d3 = dist_points(a[0], a[3]);
    vector<pair<double, int>> mins = {{d1, 1}, {d2, 2}, {d3, 3}};
    dejkstra(0, mins[0].second);
    dejkstra(2, mins[1].second);
    dejkstra(1, mins[2].second);
    cout << fixed << setprecision(3);
    cout << ans_way.size() << ' ' << ans_dist << endl;
    for (int i = 0; i < ans_way.size(); ++i) {
        cout << ans_way[i] << ' ';
    }
}
